# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from rocktick import Rocktick, AsyncRocktick
from tests.utils import assert_matches_type
from rocktick.types import (
    CronJob,
    CronListResponse,
)
from rocktick.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCron:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Rocktick) -> None:
        cron = client.cron.create(
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
            },
            schedule="schedule",
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Rocktick) -> None:
        cron = client.cron.create(
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
                "body": "body",
            },
            schedule="schedule",
            max_response_bytes=0,
            max_retries=0,
            region="region",
            timeout_ms=0,
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Rocktick) -> None:
        response = client.cron.with_raw_response.create(
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
            },
            schedule="schedule",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cron = response.parse()
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Rocktick) -> None:
        with client.cron.with_streaming_response.create(
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
            },
            schedule="schedule",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cron = response.parse()
            assert_matches_type(CronJob, cron, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Rocktick) -> None:
        cron = client.cron.retrieve(
            "job_id",
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Rocktick) -> None:
        response = client.cron.with_raw_response.retrieve(
            "job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cron = response.parse()
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Rocktick) -> None:
        with client.cron.with_streaming_response.retrieve(
            "job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cron = response.parse()
            assert_matches_type(CronJob, cron, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Rocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `job_id` but received ''"):
            client.cron.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Rocktick) -> None:
        cron = client.cron.update(
            job_id="job_id",
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Rocktick) -> None:
        cron = client.cron.update(
            job_id="job_id",
            max_response_bytes=0,
            max_retries=0,
            region="region",
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
                "body": "body",
            },
            schedule="schedule",
            timeout_ms=0,
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Rocktick) -> None:
        response = client.cron.with_raw_response.update(
            job_id="job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cron = response.parse()
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Rocktick) -> None:
        with client.cron.with_streaming_response.update(
            job_id="job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cron = response.parse()
            assert_matches_type(CronJob, cron, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Rocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `job_id` but received ''"):
            client.cron.with_raw_response.update(
                job_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Rocktick) -> None:
        cron = client.cron.list()
        assert_matches_type(SyncCursorPage[CronListResponse], cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Rocktick) -> None:
        cron = client.cron.list(
            cursor="cursor",
            limit=0,
        )
        assert_matches_type(SyncCursorPage[CronListResponse], cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Rocktick) -> None:
        response = client.cron.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cron = response.parse()
        assert_matches_type(SyncCursorPage[CronListResponse], cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Rocktick) -> None:
        with client.cron.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cron = response.parse()
            assert_matches_type(SyncCursorPage[CronListResponse], cron, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCron:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncRocktick) -> None:
        cron = await async_client.cron.create(
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
            },
            schedule="schedule",
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncRocktick) -> None:
        cron = await async_client.cron.create(
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
                "body": "body",
            },
            schedule="schedule",
            max_response_bytes=0,
            max_retries=0,
            region="region",
            timeout_ms=0,
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRocktick) -> None:
        response = await async_client.cron.with_raw_response.create(
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
            },
            schedule="schedule",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cron = await response.parse()
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRocktick) -> None:
        async with async_client.cron.with_streaming_response.create(
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
            },
            schedule="schedule",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cron = await response.parse()
            assert_matches_type(CronJob, cron, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRocktick) -> None:
        cron = await async_client.cron.retrieve(
            "job_id",
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRocktick) -> None:
        response = await async_client.cron.with_raw_response.retrieve(
            "job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cron = await response.parse()
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRocktick) -> None:
        async with async_client.cron.with_streaming_response.retrieve(
            "job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cron = await response.parse()
            assert_matches_type(CronJob, cron, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `job_id` but received ''"):
            await async_client.cron.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncRocktick) -> None:
        cron = await async_client.cron.update(
            job_id="job_id",
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncRocktick) -> None:
        cron = await async_client.cron.update(
            job_id="job_id",
            max_response_bytes=0,
            max_retries=0,
            region="region",
            request={
                "headers": {"foo": "string"},
                "method": "method",
                "url": "url",
                "body": "body",
            },
            schedule="schedule",
            timeout_ms=0,
        )
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncRocktick) -> None:
        response = await async_client.cron.with_raw_response.update(
            job_id="job_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cron = await response.parse()
        assert_matches_type(CronJob, cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncRocktick) -> None:
        async with async_client.cron.with_streaming_response.update(
            job_id="job_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cron = await response.parse()
            assert_matches_type(CronJob, cron, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncRocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `job_id` but received ''"):
            await async_client.cron.with_raw_response.update(
                job_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncRocktick) -> None:
        cron = await async_client.cron.list()
        assert_matches_type(AsyncCursorPage[CronListResponse], cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncRocktick) -> None:
        cron = await async_client.cron.list(
            cursor="cursor",
            limit=0,
        )
        assert_matches_type(AsyncCursorPage[CronListResponse], cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncRocktick) -> None:
        response = await async_client.cron.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        cron = await response.parse()
        assert_matches_type(AsyncCursorPage[CronListResponse], cron, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncRocktick) -> None:
        async with async_client.cron.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            cron = await response.parse()
            assert_matches_type(AsyncCursorPage[CronListResponse], cron, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["VerifyRetrieveResponse"]


class VerifyRetrieveResponse(BaseModel):
    verified: bool

    hash: Optional[str] = None
    """The hex digest of the request body, or null if there isn't one for the request."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import cron_list_params, cron_create_params, cron_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorPage, AsyncCursorPage
from .._base_client import AsyncPaginator, make_request_options
from ..types.cron_job import CronJob
from ..types.cron_list_response import CronListResponse
from ..types.http_request_param import HTTPRequestParam

__all__ = ["CronResource", "AsyncCronResource"]


class CronResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CronResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/violetbuse/rocktick-python#accessing-raw-response-data-eg-headers
        """
        return CronResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CronResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/violetbuse/rocktick-python#with_streaming_response
        """
        return CronResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        request: HTTPRequestParam,
        schedule: str,
        max_response_bytes: Optional[int] | Omit = omit,
        max_retries: Optional[int] | Omit = omit,
        region: Optional[str] | Omit = omit,
        timeout_ms: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CronJob:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/cron",
            body=maybe_transform(
                {
                    "request": request,
                    "schedule": schedule,
                    "max_response_bytes": max_response_bytes,
                    "max_retries": max_retries,
                    "region": region,
                    "timeout_ms": timeout_ms,
                },
                cron_create_params.CronCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CronJob,
        )

    def retrieve(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CronJob:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            f"/api/cron/{job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CronJob,
        )

    def update(
        self,
        job_id: str,
        *,
        max_response_bytes: Optional[int] | Omit = omit,
        max_retries: Optional[int] | Omit = omit,
        region: Optional[str] | Omit = omit,
        request: Optional[HTTPRequestParam] | Omit = omit,
        schedule: Optional[str] | Omit = omit,
        timeout_ms: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CronJob:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._post(
            f"/api/cron/{job_id}",
            body=maybe_transform(
                {
                    "max_response_bytes": max_response_bytes,
                    "max_retries": max_retries,
                    "region": region,
                    "request": request,
                    "schedule": schedule,
                    "timeout_ms": timeout_ms,
                },
                cron_update_params.CronUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CronJob,
        )

    def list(
        self,
        *,
        cursor: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[CronListResponse]:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/cron",
            page=SyncCursorPage[CronListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    cron_list_params.CronListParams,
                ),
            ),
            model=CronListResponse,
        )


class AsyncCronResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCronResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/violetbuse/rocktick-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCronResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCronResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/violetbuse/rocktick-python#with_streaming_response
        """
        return AsyncCronResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        request: HTTPRequestParam,
        schedule: str,
        max_response_bytes: Optional[int] | Omit = omit,
        max_retries: Optional[int] | Omit = omit,
        region: Optional[str] | Omit = omit,
        timeout_ms: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CronJob:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/cron",
            body=await async_maybe_transform(
                {
                    "request": request,
                    "schedule": schedule,
                    "max_response_bytes": max_response_bytes,
                    "max_retries": max_retries,
                    "region": region,
                    "timeout_ms": timeout_ms,
                },
                cron_create_params.CronCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CronJob,
        )

    async def retrieve(
        self,
        job_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CronJob:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            f"/api/cron/{job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CronJob,
        )

    async def update(
        self,
        job_id: str,
        *,
        max_response_bytes: Optional[int] | Omit = omit,
        max_retries: Optional[int] | Omit = omit,
        region: Optional[str] | Omit = omit,
        request: Optional[HTTPRequestParam] | Omit = omit,
        schedule: Optional[str] | Omit = omit,
        timeout_ms: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CronJob:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._post(
            f"/api/cron/{job_id}",
            body=await async_maybe_transform(
                {
                    "max_response_bytes": max_response_bytes,
                    "max_retries": max_retries,
                    "region": region,
                    "request": request,
                    "schedule": schedule,
                    "timeout_ms": timeout_ms,
                },
                cron_update_params.CronUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CronJob,
        )

    def list(
        self,
        *,
        cursor: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CronListResponse, AsyncCursorPage[CronListResponse]]:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/cron",
            page=AsyncCursorPage[CronListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    cron_list_params.CronListParams,
                ),
            ),
            model=CronListResponse,
        )


class CronResourceWithRawResponse:
    def __init__(self, cron: CronResource) -> None:
        self._cron = cron

        self.create = to_raw_response_wrapper(
            cron.create,
        )
        self.retrieve = to_raw_response_wrapper(
            cron.retrieve,
        )
        self.update = to_raw_response_wrapper(
            cron.update,
        )
        self.list = to_raw_response_wrapper(
            cron.list,
        )


class AsyncCronResourceWithRawResponse:
    def __init__(self, cron: AsyncCronResource) -> None:
        self._cron = cron

        self.create = async_to_raw_response_wrapper(
            cron.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            cron.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            cron.update,
        )
        self.list = async_to_raw_response_wrapper(
            cron.list,
        )


class CronResourceWithStreamingResponse:
    def __init__(self, cron: CronResource) -> None:
        self._cron = cron

        self.create = to_streamed_response_wrapper(
            cron.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            cron.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            cron.update,
        )
        self.list = to_streamed_response_wrapper(
            cron.list,
        )


class AsyncCronResourceWithStreamingResponse:
    def __init__(self, cron: AsyncCronResource) -> None:
        self._cron = cron

        self.create = async_to_streamed_response_wrapper(
            cron.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            cron.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            cron.update,
        )
        self.list = async_to_streamed_response_wrapper(
            cron.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .cron import (
    CronResource,
    AsyncCronResource,
    CronResourceWithRawResponse,
    AsyncCronResourceWithRawResponse,
    CronResourceWithStreamingResponse,
    AsyncCronResourceWithStreamingResponse,
)
from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .verify import (
    VerifyResource,
    AsyncVerifyResource,
    VerifyResourceWithRawResponse,
    AsyncVerifyResourceWithRawResponse,
    VerifyResourceWithStreamingResponse,
    AsyncVerifyResourceWithStreamingResponse,
)
from .tenants import (
    TenantsResource,
    AsyncTenantsResource,
    TenantsResourceWithRawResponse,
    AsyncTenantsResourceWithRawResponse,
    TenantsResourceWithStreamingResponse,
    AsyncTenantsResourceWithStreamingResponse,
)
from .executions import (
    ExecutionsResource,
    AsyncExecutionsResource,
    ExecutionsResourceWithRawResponse,
    AsyncExecutionsResourceWithRawResponse,
    ExecutionsResourceWithStreamingResponse,
    AsyncExecutionsResourceWithStreamingResponse,
)

__all__ = [
    "CronResource",
    "AsyncCronResource",
    "CronResourceWithRawResponse",
    "AsyncCronResourceWithRawResponse",
    "CronResourceWithStreamingResponse",
    "AsyncCronResourceWithStreamingResponse",
    "ExecutionsResource",
    "AsyncExecutionsResource",
    "ExecutionsResourceWithRawResponse",
    "AsyncExecutionsResourceWithRawResponse",
    "ExecutionsResourceWithStreamingResponse",
    "AsyncExecutionsResourceWithStreamingResponse",
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "TenantsResource",
    "AsyncTenantsResource",
    "TenantsResourceWithRawResponse",
    "AsyncTenantsResourceWithRawResponse",
    "TenantsResourceWithStreamingResponse",
    "AsyncTenantsResourceWithStreamingResponse",
    "VerifyResource",
    "AsyncVerifyResource",
    "VerifyResourceWithRawResponse",
    "AsyncVerifyResourceWithRawResponse",
    "VerifyResourceWithStreamingResponse",
    "AsyncVerifyResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TenantCreateParams"]


class TenantCreateParams(TypedDict, total=False):
    default_retries: Required[int]

    max_max_response_bytes: Required[int]

    max_request_bytes: Required[int]

    max_timeout: Required[int]

    max_tokens: Required[int]

    tok_per_day: Required[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .execution import Execution
from .http_request import HTTPRequest

__all__ = ["OneOffJob"]


class OneOffJob(BaseModel):
    id: str

    execute_at: int

    executions: List[Execution]

    max_retries: int

    region: str

    request: HTTPRequest

    max_response_bytes: Optional[int] = None

    tenant_id: Optional[str] = None

    timeout_ms: Optional[int] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..types import tenant_create_params, tenant_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.tenant import Tenant

__all__ = ["TenantsResource", "AsyncTenantsResource"]


class TenantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/violetbuse/rocktick-python#accessing-raw-response-data-eg-headers
        """
        return TenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/violetbuse/rocktick-python#with_streaming_response
        """
        return TenantsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        default_retries: int,
        max_max_response_bytes: int,
        max_request_bytes: int,
        max_timeout: int,
        max_tokens: int,
        tok_per_day: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/tenants",
            body=maybe_transform(
                {
                    "default_retries": default_retries,
                    "max_max_response_bytes": max_max_response_bytes,
                    "max_request_bytes": max_request_bytes,
                    "max_timeout": max_timeout,
                    "max_tokens": max_tokens,
                    "tok_per_day": tok_per_day,
                },
                tenant_create_params.TenantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    def retrieve(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._get(
            f"/api/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    def update(
        self,
        tenant_id: str,
        *,
        default_retries: Optional[int] | Omit = omit,
        max_max_response_bytes: Optional[int] | Omit = omit,
        max_request_bytes: Optional[int] | Omit = omit,
        max_timeout: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        tok_per_day: Optional[int] | Omit = omit,
        tokens: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return self._post(
            f"/api/tenants/{tenant_id}",
            body=maybe_transform(
                {
                    "default_retries": default_retries,
                    "max_max_response_bytes": max_max_response_bytes,
                    "max_request_bytes": max_request_bytes,
                    "max_timeout": max_timeout,
                    "max_tokens": max_tokens,
                    "tok_per_day": tok_per_day,
                    "tokens": tokens,
                },
                tenant_update_params.TenantUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )


class AsyncTenantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/violetbuse/rocktick-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/violetbuse/rocktick-python#with_streaming_response
        """
        return AsyncTenantsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        default_retries: int,
        max_max_response_bytes: int,
        max_request_bytes: int,
        max_timeout: int,
        max_tokens: int,
        tok_per_day: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/tenants",
            body=await async_maybe_transform(
                {
                    "default_retries": default_retries,
                    "max_max_response_bytes": max_max_response_bytes,
                    "max_request_bytes": max_request_bytes,
                    "max_timeout": max_timeout,
                    "max_tokens": max_tokens,
                    "tok_per_day": tok_per_day,
                },
                tenant_create_params.TenantCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    async def retrieve(
        self,
        tenant_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._get(
            f"/api/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )

    async def update(
        self,
        tenant_id: str,
        *,
        default_retries: Optional[int] | Omit = omit,
        max_max_response_bytes: Optional[int] | Omit = omit,
        max_request_bytes: Optional[int] | Omit = omit,
        max_timeout: Optional[int] | Omit = omit,
        max_tokens: Optional[int] | Omit = omit,
        tok_per_day: Optional[int] | Omit = omit,
        tokens: Optional[int] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Tenant:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        return await self._post(
            f"/api/tenants/{tenant_id}",
            body=await async_maybe_transform(
                {
                    "default_retries": default_retries,
                    "max_max_response_bytes": max_max_response_bytes,
                    "max_request_bytes": max_request_bytes,
                    "max_timeout": max_timeout,
                    "max_tokens": max_tokens,
                    "tok_per_day": tok_per_day,
                    "tokens": tokens,
                },
                tenant_update_params.TenantUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Tenant,
        )


class TenantsResourceWithRawResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.create = to_raw_response_wrapper(
            tenants.create,
        )
        self.retrieve = to_raw_response_wrapper(
            tenants.retrieve,
        )
        self.update = to_raw_response_wrapper(
            tenants.update,
        )


class AsyncTenantsResourceWithRawResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.create = async_to_raw_response_wrapper(
            tenants.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            tenants.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            tenants.update,
        )


class TenantsResourceWithStreamingResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.create = to_streamed_response_wrapper(
            tenants.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            tenants.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            tenants.update,
        )


class AsyncTenantsResourceWithStreamingResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.create = async_to_streamed_response_wrapper(
            tenants.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            tenants.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            tenants.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from rocktick import Rocktick, AsyncRocktick
from tests.utils import assert_matches_type
from rocktick.types import Tenant

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTenants:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Rocktick) -> None:
        tenant = client.tenants.create(
            default_retries=0,
            max_max_response_bytes=0,
            max_request_bytes=0,
            max_timeout=0,
            max_tokens=0,
            tok_per_day=0,
        )
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Rocktick) -> None:
        response = client.tenants.with_raw_response.create(
            default_retries=0,
            max_max_response_bytes=0,
            max_request_bytes=0,
            max_timeout=0,
            max_tokens=0,
            tok_per_day=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = response.parse()
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Rocktick) -> None:
        with client.tenants.with_streaming_response.create(
            default_retries=0,
            max_max_response_bytes=0,
            max_request_bytes=0,
            max_timeout=0,
            max_tokens=0,
            tok_per_day=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = response.parse()
            assert_matches_type(Tenant, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Rocktick) -> None:
        tenant = client.tenants.retrieve(
            "tenant_id",
        )
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Rocktick) -> None:
        response = client.tenants.with_raw_response.retrieve(
            "tenant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = response.parse()
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Rocktick) -> None:
        with client.tenants.with_streaming_response.retrieve(
            "tenant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = response.parse()
            assert_matches_type(Tenant, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Rocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            client.tenants.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Rocktick) -> None:
        tenant = client.tenants.update(
            tenant_id="tenant_id",
        )
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Rocktick) -> None:
        tenant = client.tenants.update(
            tenant_id="tenant_id",
            default_retries=0,
            max_max_response_bytes=0,
            max_request_bytes=0,
            max_timeout=0,
            max_tokens=0,
            tok_per_day=0,
            tokens=0,
        )
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Rocktick) -> None:
        response = client.tenants.with_raw_response.update(
            tenant_id="tenant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = response.parse()
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Rocktick) -> None:
        with client.tenants.with_streaming_response.update(
            tenant_id="tenant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = response.parse()
            assert_matches_type(Tenant, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Rocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            client.tenants.with_raw_response.update(
                tenant_id="",
            )


class TestAsyncTenants:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncRocktick) -> None:
        tenant = await async_client.tenants.create(
            default_retries=0,
            max_max_response_bytes=0,
            max_request_bytes=0,
            max_timeout=0,
            max_tokens=0,
            tok_per_day=0,
        )
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncRocktick) -> None:
        response = await async_client.tenants.with_raw_response.create(
            default_retries=0,
            max_max_response_bytes=0,
            max_request_bytes=0,
            max_timeout=0,
            max_tokens=0,
            tok_per_day=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = await response.parse()
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncRocktick) -> None:
        async with async_client.tenants.with_streaming_response.create(
            default_retries=0,
            max_max_response_bytes=0,
            max_request_bytes=0,
            max_timeout=0,
            max_tokens=0,
            tok_per_day=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = await response.parse()
            assert_matches_type(Tenant, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRocktick) -> None:
        tenant = await async_client.tenants.retrieve(
            "tenant_id",
        )
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRocktick) -> None:
        response = await async_client.tenants.with_raw_response.retrieve(
            "tenant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = await response.parse()
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRocktick) -> None:
        async with async_client.tenants.with_streaming_response.retrieve(
            "tenant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = await response.parse()
            assert_matches_type(Tenant, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            await async_client.tenants.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncRocktick) -> None:
        tenant = await async_client.tenants.update(
            tenant_id="tenant_id",
        )
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncRocktick) -> None:
        tenant = await async_client.tenants.update(
            tenant_id="tenant_id",
            default_retries=0,
            max_max_response_bytes=0,
            max_request_bytes=0,
            max_timeout=0,
            max_tokens=0,
            tok_per_day=0,
            tokens=0,
        )
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncRocktick) -> None:
        response = await async_client.tenants.with_raw_response.update(
            tenant_id="tenant_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = await response.parse()
        assert_matches_type(Tenant, tenant, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncRocktick) -> None:
        async with async_client.tenants.with_streaming_response.update(
            tenant_id="tenant_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = await response.parse()
            assert_matches_type(Tenant, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncRocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_id` but received ''"):
            await async_client.tenants.with_raw_response.update(
                tenant_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict

from .._models import BaseModel

__all__ = ["HTTPResponse"]


class HTTPResponse(BaseModel):
    body: str

    headers: Dict[str, str]

    status: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from rocktick import Rocktick, AsyncRocktick
from tests.utils import assert_matches_type
from rocktick.types import Execution, ExecutionListResponse
from rocktick.pagination import SyncCursorPage, AsyncCursorPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestExecutions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Rocktick) -> None:
        execution = client.executions.retrieve(
            "execution_id",
        )
        assert_matches_type(Execution, execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Rocktick) -> None:
        response = client.executions.with_raw_response.retrieve(
            "execution_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        execution = response.parse()
        assert_matches_type(Execution, execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Rocktick) -> None:
        with client.executions.with_streaming_response.retrieve(
            "execution_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            execution = response.parse()
            assert_matches_type(Execution, execution, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Rocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `execution_id` but received ''"):
            client.executions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Rocktick) -> None:
        execution = client.executions.list()
        assert_matches_type(SyncCursorPage[ExecutionListResponse], execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Rocktick) -> None:
        execution = client.executions.list(
            completed=True,
            cron_id="cron_id",
            cursor="cursor",
            from_=0,
            limit=0,
            one_off_job_id="one_off_job_id",
            to=0,
        )
        assert_matches_type(SyncCursorPage[ExecutionListResponse], execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Rocktick) -> None:
        response = client.executions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        execution = response.parse()
        assert_matches_type(SyncCursorPage[ExecutionListResponse], execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Rocktick) -> None:
        with client.executions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            execution = response.parse()
            assert_matches_type(SyncCursorPage[ExecutionListResponse], execution, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncExecutions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncRocktick) -> None:
        execution = await async_client.executions.retrieve(
            "execution_id",
        )
        assert_matches_type(Execution, execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncRocktick) -> None:
        response = await async_client.executions.with_raw_response.retrieve(
            "execution_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        execution = await response.parse()
        assert_matches_type(Execution, execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncRocktick) -> None:
        async with async_client.executions.with_streaming_response.retrieve(
            "execution_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            execution = await response.parse()
            assert_matches_type(Execution, execution, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncRocktick) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `execution_id` but received ''"):
            await async_client.executions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncRocktick) -> None:
        execution = await async_client.executions.list()
        assert_matches_type(AsyncCursorPage[ExecutionListResponse], execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncRocktick) -> None:
        execution = await async_client.executions.list(
            completed=True,
            cron_id="cron_id",
            cursor="cursor",
            from_=0,
            limit=0,
            one_off_job_id="one_off_job_id",
            to=0,
        )
        assert_matches_type(AsyncCursorPage[ExecutionListResponse], execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncRocktick) -> None:
        response = await async_client.executions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        execution = await response.parse()
        assert_matches_type(AsyncCursorPage[ExecutionListResponse], execution, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncRocktick) -> None:
        async with async_client.executions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            execution = await response.parse()
            assert_matches_type(AsyncCursorPage[ExecutionListResponse], execution, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["TenantUpdateParams"]


class TenantUpdateParams(TypedDict, total=False):
    default_retries: Optional[int]

    max_max_response_bytes: Optional[int]

    max_request_bytes: Optional[int]

    max_timeout: Optional[int]

    max_tokens: Optional[int]

    tok_per_day: Optional[int]

    tokens: Optional[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .http_request_param import HTTPRequestParam

__all__ = ["JobCreateParams"]


class JobCreateParams(TypedDict, total=False):
    execute_at: Required[int]

    request: Required[HTTPRequestParam]

    max_response_bytes: Optional[int]

    max_retries: Optional[int]

    region: Optional[str]

    timeout_ms: Optional[int]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Dict, Mapping, cast
from typing_extensions import Self, Literal, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import RocktickError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import cron, jobs, verify, tenants, executions
    from .resources.cron import CronResource, AsyncCronResource
    from .resources.jobs import JobsResource, AsyncJobsResource
    from .resources.verify import VerifyResource, AsyncVerifyResource
    from .resources.tenants import TenantsResource, AsyncTenantsResource
    from .resources.executions import ExecutionsResource, AsyncExecutionsResource

__all__ = [
    "ENVIRONMENTS",
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Rocktick",
    "AsyncRocktick",
    "Client",
    "AsyncClient",
]

ENVIRONMENTS: Dict[str, str] = {
    "production": "https://rocktick.com",
    "local": "http://localhost:9090",
}


class Rocktick(SyncAPIClient):
    # client options
    api_key: str
    tenant_id: str | None

    _environment: Literal["production", "local"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        tenant_id: str | None = None,
        environment: Literal["production", "local"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Rocktick client instance.

        This automatically infers the `api_key` argument from the `ROCKTICK_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ROCKTICK_API_KEY")
        if api_key is None:
            raise RocktickError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ROCKTICK_API_KEY environment variable"
            )
        self.api_key = api_key

        self.tenant_id = tenant_id

        self._environment = environment

        base_url_env = os.environ.get("ROCKTICK_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `ROCKTICK_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def cron(self) -> CronResource:
        from .resources.cron import CronResource

        return CronResource(self)

    @cached_property
    def executions(self) -> ExecutionsResource:
        from .resources.executions import ExecutionsResource

        return ExecutionsResource(self)

    @cached_property
    def jobs(self) -> JobsResource:
        from .resources.jobs import JobsResource

        return JobsResource(self)

    @cached_property
    def tenants(self) -> TenantsResource:
        from .resources.tenants import TenantsResource

        return TenantsResource(self)

    @cached_property
    def verify(self) -> VerifyResource:
        from .resources.verify import VerifyResource

        return VerifyResource(self)

    @cached_property
    def with_raw_response(self) -> RocktickWithRawResponse:
        return RocktickWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RocktickWithStreamedResponse:
        return RocktickWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "Tenant-Id": self.tenant_id if self.tenant_id is not None else Omit(),
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        tenant_id: str | None = None,
        environment: Literal["production", "local"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            tenant_id=tenant_id or self.tenant_id,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncRocktick(AsyncAPIClient):
    # client options
    api_key: str
    tenant_id: str | None

    _environment: Literal["production", "local"] | NotGiven

    def __init__(
        self,
        *,
        api_key: str | None = None,
        tenant_id: str | None = None,
        environment: Literal["production", "local"] | NotGiven = not_given,
        base_url: str | httpx.URL | None | NotGiven = not_given,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncRocktick client instance.

        This automatically infers the `api_key` argument from the `ROCKTICK_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("ROCKTICK_API_KEY")
        if api_key is None:
            raise RocktickError(
                "The api_key client option must be set either by passing api_key to the client or by setting the ROCKTICK_API_KEY environment variable"
            )
        self.api_key = api_key

        self.tenant_id = tenant_id

        self._environment = environment

        base_url_env = os.environ.get("ROCKTICK_BASE_URL")
        if is_given(base_url) and base_url is not None:
            # cast required because mypy doesn't understand the type narrowing
            base_url = cast("str | httpx.URL", base_url)  # pyright: ignore[reportUnnecessaryCast]
        elif is_given(environment):
            if base_url_env and base_url is not None:
                raise ValueError(
                    "Ambiguous URL; The `ROCKTICK_BASE_URL` env var and the `environment` argument are given. If you want to use the environment, you must pass base_url=None",
                )

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc
        elif base_url_env is not None:
            base_url = base_url_env
        else:
            self._environment = environment = "production"

            try:
                base_url = ENVIRONMENTS[environment]
            except KeyError as exc:
                raise ValueError(f"Unknown environment: {environment}") from exc

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def cron(self) -> AsyncCronResource:
        from .resources.cron import AsyncCronResource

        return AsyncCronResource(self)

    @cached_property
    def executions(self) -> AsyncExecutionsResource:
        from .resources.executions import AsyncExecutionsResource

        return AsyncExecutionsResource(self)

    @cached_property
    def jobs(self) -> AsyncJobsResource:
        from .resources.jobs import AsyncJobsResource

        return AsyncJobsResource(self)

    @cached_property
    def tenants(self) -> AsyncTenantsResource:
        from .resources.tenants import AsyncTenantsResource

        return AsyncTenantsResource(self)

    @cached_property
    def verify(self) -> AsyncVerifyResource:
        from .resources.verify import AsyncVerifyResource

        return AsyncVerifyResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncRocktickWithRawResponse:
        return AsyncRocktickWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRocktickWithStreamedResponse:
        return AsyncRocktickWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "Tenant-Id": self.tenant_id if self.tenant_id is not None else Omit(),
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        tenant_id: str | None = None,
        environment: Literal["production", "local"] | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            tenant_id=tenant_id or self.tenant_id,
            base_url=base_url or self.base_url,
            environment=environment or self._environment,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class RocktickWithRawResponse:
    _client: Rocktick

    def __init__(self, client: Rocktick) -> None:
        self._client = client

    @cached_property
    def cron(self) -> cron.CronResourceWithRawResponse:
        from .resources.cron import CronResourceWithRawResponse

        return CronResourceWithRawResponse(self._client.cron)

    @cached_property
    def executions(self) -> executions.ExecutionsResourceWithRawResponse:
        from .resources.executions import ExecutionsResourceWithRawResponse

        return ExecutionsResourceWithRawResponse(self._client.executions)

    @cached_property
    def jobs(self) -> jobs.JobsResourceWithRawResponse:
        from .resources.jobs import JobsResourceWithRawResponse

        return JobsResourceWithRawResponse(self._client.jobs)

    @cached_property
    def tenants(self) -> tenants.TenantsResourceWithRawResponse:
        from .resources.tenants import TenantsResourceWithRawResponse

        return TenantsResourceWithRawResponse(self._client.tenants)

    @cached_property
    def verify(self) -> verify.VerifyResourceWithRawResponse:
        from .resources.verify import VerifyResourceWithRawResponse

        return VerifyResourceWithRawResponse(self._client.verify)


class AsyncRocktickWithRawResponse:
    _client: AsyncRocktick

    def __init__(self, client: AsyncRocktick) -> None:
        self._client = client

    @cached_property
    def cron(self) -> cron.AsyncCronResourceWithRawResponse:
        from .resources.cron import AsyncCronResourceWithRawResponse

        return AsyncCronResourceWithRawResponse(self._client.cron)

    @cached_property
    def executions(self) -> executions.AsyncExecutionsResourceWithRawResponse:
        from .resources.executions import AsyncExecutionsResourceWithRawResponse

        return AsyncExecutionsResourceWithRawResponse(self._client.executions)

    @cached_property
    def jobs(self) -> jobs.AsyncJobsResourceWithRawResponse:
        from .resources.jobs import AsyncJobsResourceWithRawResponse

        return AsyncJobsResourceWithRawResponse(self._client.jobs)

    @cached_property
    def tenants(self) -> tenants.AsyncTenantsResourceWithRawResponse:
        from .resources.tenants import AsyncTenantsResourceWithRawResponse

        return AsyncTenantsResourceWithRawResponse(self._client.tenants)

    @cached_property
    def verify(self) -> verify.AsyncVerifyResourceWithRawResponse:
        from .resources.verify import AsyncVerifyResourceWithRawResponse

        return AsyncVerifyResourceWithRawResponse(self._client.verify)


class RocktickWithStreamedResponse:
    _client: Rocktick

    def __init__(self, client: Rocktick) -> None:
        self._client = client

    @cached_property
    def cron(self) -> cron.CronResourceWithStreamingResponse:
        from .resources.cron import CronResourceWithStreamingResponse

        return CronResourceWithStreamingResponse(self._client.cron)

    @cached_property
    def executions(self) -> executions.ExecutionsResourceWithStreamingResponse:
        from .resources.executions import ExecutionsResourceWithStreamingResponse

        return ExecutionsResourceWithStreamingResponse(self._client.executions)

    @cached_property
    def jobs(self) -> jobs.JobsResourceWithStreamingResponse:
        from .resources.jobs import JobsResourceWithStreamingResponse

        return JobsResourceWithStreamingResponse(self._client.jobs)

    @cached_property
    def tenants(self) -> tenants.TenantsResourceWithStreamingResponse:
        from .resources.tenants import TenantsResourceWithStreamingResponse

        return TenantsResourceWithStreamingResponse(self._client.tenants)

    @cached_property
    def verify(self) -> verify.VerifyResourceWithStreamingResponse:
        from .resources.verify import VerifyResourceWithStreamingResponse

        return VerifyResourceWithStreamingResponse(self._client.verify)


class AsyncRocktickWithStreamedResponse:
    _client: AsyncRocktick

    def __init__(self, client: AsyncRocktick) -> None:
        self._client = client

    @cached_property
    def cron(self) -> cron.AsyncCronResourceWithStreamingResponse:
        from .resources.cron import AsyncCronResourceWithStreamingResponse

        return AsyncCronResourceWithStreamingResponse(self._client.cron)

    @cached_property
    def executions(self) -> executions.AsyncExecutionsResourceWithStreamingResponse:
        from .resources.executions import AsyncExecutionsResourceWithStreamingResponse

        return AsyncExecutionsResourceWithStreamingResponse(self._client.executions)

    @cached_property
    def jobs(self) -> jobs.AsyncJobsResourceWithStreamingResponse:
        from .resources.jobs import AsyncJobsResourceWithStreamingResponse

        return AsyncJobsResourceWithStreamingResponse(self._client.jobs)

    @cached_property
    def tenants(self) -> tenants.AsyncTenantsResourceWithStreamingResponse:
        from .resources.tenants import AsyncTenantsResourceWithStreamingResponse

        return AsyncTenantsResourceWithStreamingResponse(self._client.tenants)

    @cached_property
    def verify(self) -> verify.AsyncVerifyResourceWithStreamingResponse:
        from .resources.verify import AsyncVerifyResourceWithStreamingResponse

        return AsyncVerifyResourceWithStreamingResponse(self._client.verify)


Client = Rocktick

AsyncClient = AsyncRocktick
